//The research was supported by the project  GACR GA15-03564S

// Simulation of two dimensional static categorical mixture 
//                  with dynamic data-dependent pointer
//  - structure
//  pointer     f(c|c1,al)   al: matrix 3x3
//  components  f(y1,y2|c,par_c)  par: table 3x4  with head
//                                     y1,y2=1,1  1,2  2,1  2,2
//                                     c=1  |    |    |    |
//                                          ----------------
//                                     c=2  |    |    |    |
//                                          ----------------
//                                     c=3  |    |    |    |
// -------------------------------------------------------------------------
exec("Intro.sce",-1), mode(0)

nd=1500;                 // number of data
ky=[2 2];                // number of values of y(1,:) and y(2,:)
nc=3;                    // number of components
kz=3;                                   // no. of values of zt

// external variable zt 
zt=genz(kz,nd);

pp=list();
pp(1)=fnorm([0 0 1; 1 0 0; 0 1 0]+.05,2); 
pp(2)=fnorm([1 0 0; 0 1 0; 0 0 1]+.1,2);
pp(3)=fnorm([0 1 0; 0 0 1; 1 0 0]+.08,2); 

al=list();
for i=1:kz
  al(i)=fnorm(pp(i)(1:nc,1:nc),2);
end

// parameters of components  
p=list();
par=[130 3 4 2]; p(1)=par/sum(par);     // the length of parameters
par=[2 120 1 3]; p(2)=par/sum(par);     //  must be equal to
par=[1 1 3 140]; p(3)=par/sum(par);     //  prod(ky) !!!!
par=[1 1 100 1]; p(4)=par/sum(par); 
par=[1 200 3 8]; p(5)=par/sum(par); 


c1=2;
ct=ones(1,nd);
yt=ones(2,nd);

// SIMULATION
for t=2:nd
  pr=al(zt(t))(c1,:);
  c=sum(rand(1,1,'unif')>cumsum(pr))+1; // generation of active component
  yy=sum(rand(1,1,'unif')>cumsum(p(c)))+1; // generation of data from act.comp.
  yt(:,t)=row2psi(yy,ky)';              // transformation to y(1) and y(2)
  c1=c;                                 // remember as past component
  ct(t)=c;                              // remember for graph
end

// generation of prior classified data
yi=[]; ni=5;                            // no. of data from each comp.
for i=1:nc
  j=find(ct==i);                        // data from component i
  ji=j(10+(1:ni));                      // selected from data sample
  yi=[yi [yt(:,ji); i*ones(1,ni)]];     // stored in yi
end

// RESULTS
scf(1);
set(gcf(),'background',8);
plot(yt(1,:)+.01*rand(1,nd,'norm'),yt(2,:)+.01*rand(1,nd,'norm'),'.:')
title('Data clusters','fontsize',4)

scf(2);
plot(1:nd,ct,'.')
title('Component switching','fontsize',4)
set(gca(),'data_bounds',[1 nd .9 nc+.1])

// SAVING THE DATA SAMPLE (the name is compulsory)
als=al; ps=p; ncs=nc; kys=ky;
save('_data/datDisEx','yt','zt','ct','yi','als','ps','ncs','kys')
