//The research was supported by the project  GACR GA15-03564S

//// Simulation of two dimensional state-space mixture 
////                  with dynamic data-dependent pointer
// -------------------------------------------------------------------------
exec("Intro.sce",-1)

nd=1500;                                // number of data
nc=3;                                   // number of components <= 5
cF=.5;                                  // proximity of initial components
kz=3;                                   // no. of values of zt

// external variable zt 
zt=genz(kz,nd);

pp=list();
pp(1)=fnorm([0 0 1; 1 0 0; 0 1 0]+.05,2); 
pp(2)=fnorm([1 0 0; 0 1 0; 0 0 1]+.1,2);
pp(3)=fnorm([0 1 0; 0 0 1; 1 0 0]+.08,2); 

als=list();
for i=1:kz
  als(i)=fnorm(pp(i)(1:nc,1:nc),2);
end

M=list(); A=list(); F=list(); Rw=list(); Rv=list(); 
// component model parameters
M(1)=[.3 -.3; .5 .1]; A(1)=[.9 .4;1 -.4]; F(1)=cF*[-1 2]';
Rw(1)=[1 0; 0 1]*.1; Rv(1)=[.1 0;0 .1]*.1;

M(2)=[.05 -.3; .4 -.1]; A(2)=[2.1 2.8;1 -.3]; F(2)=cF*[3 1]';
Rw(2)=[1 0; 0 1]*.1; Rv(2)=[.2 0;0 .2]*.1;

M(3)=[.6 -.1; .8 -.5]; A(3)=[1.9 -.4;1 .5]; F(3)=cF*[-3 -2]';
Rw(3)=[1 0; 0 1]*.1; Rv(3)=[.5 0;0 .3]*.1;

M(4)=[.2 -.5; .2 -.9]; A(4)=[.9 -.8;1 1.5]; F(4)=cF*[3 -2]';
Rw(4)=[.1 0; 0 .1]*.1; Rv(4)=[.05 0;0 .8]*.1;

M(5)=[.9 -.01; .08 -.3]; A(5)=[2.9 -.04;.1 1.5]; F(5)=cF*[-1 3]';
Rw(5)=[.1 0; 0 1]*.1; Rv(5)=[1.5 0;0 .6]*.1;
   
x=zeros(2,nd);                          // declarations
yt=zeros(2,nd);
ct=zeros(1,nd);

ct(1)=1;                                // initial past component
// SIMULACE
for t=2:nd
  c=sum(rand(1,1,'unif')>cumsum(als(zt(t))(ct(t-1),:)))+1; // generation of act. comp.
  x(:,t)=M(c)*x(:,t-1)+F(c)+Rw(c)*rand(2,1,'norm'); // generation of state
  yt(:,t)=A(c)*x(:,t)+Rv(c)*rand(2,1,'norm');       // generation of output
  ct(t)=c;                              // stor for graph
end

// RESULTS
cla
set(scf(1),'position',[600 50 500 500])
title('Output and state variables','fontsize',4)
subplot(211)
plot(yt')
xlabel('time','fontsize',4)
ylabel('variables','fontsize',4)
set(gca(),'margins',[0.2,0.2,0.2,0.2])
subplot(212)
plot(x')
xlabel('time','fontsize',4)
ylabel('variables','fontsize',4)
set(gca(),'margins',[0.2,0.2,0.2,0.2])

set(scf(2),'position',[700 100 500 500])
title('Output and state clusters','fontsize',4)
subplot(211)
plot(yt(1,:),yt(2,:),'.')
xlabel('first variable','fontsize',4)
ylabel('second variable','fontsize',4)
set(gca(),'margins',[0.2,0.2,0.2,0.2])
subplot(212)
plot(x(1,:),x(2,:),'.')
xlabel('first variable','fontsize',4)
ylabel('second variable','fontsize',4)
set(gca(),'margins',[0.2,0.2,0.2,0.2])

set(scf(3),'position',[100 50 500 500])
title('Values of the pointer','fontsize',4)
plot(ct,'.')
xlabel('time','fontsize',4)
ylabel('pointer values','fontsize',4)
set(gca(),'margins',[0.2,0.2,0.2,0.2])
set(gca(),'data_bounds',[1 nd .9 nc+.1])

// SAVING THE DATA SAMPLE (the name is compulsory)
ncs=nc;
save('_data/datStaEx','yt','zt','ct','als','ncs','M','A','F','Rw','Rv')

