//The research was supported by the project  GACR GA15-03564S

//// Simulation of two dimensional static exponential mixture 
////                  with dynamic data-dependent pointer
// -------------------------------------------------------------------------
// basic simulation examples
mode(0),funcprot(0);
exec("Intro.sce",-1)

nd=1500;                                // number of data
nc=3;                                   // number of components
ny=2;                                   // dimension of the output
kz=3;                                   // no. of values of zt

// external variable zt 
zt=genz(kz,nd);

pp=list();
pp(1)=fnorm([0 0 1; 1 0 0; 0 1 0]+.05,2); 
pp(2)=fnorm([1 0 0; 0 1 0; 0 0 1]+.1,2);
pp(3)=fnorm([0 1 0; 0 0 1; 1 0 0]+.08,2); 

// here you can define your own component parameters:
ths=list(); as=list();
ths(1)=[.2 9]';     as(1)=[1 19]';
ths(2)=[1 1]';      as(2)=[0 0]';
ths(3)=[5 .1]';     as(3)=[16 1]';
ths(4)=[.5 .4]';    as(4)=[3 9]';
ths(5)=[.5 1]';     as(5)=[16 3]';
// --------------------------------------
ct(1)=1;                                // initial component
// SIMULATION
for t=2:nd
  cp=cumsum(pp(zt(t)),2);
  ct(t)=sum(rand(1,1,'unif')>cp(ct(t-1),:))+1;      // component generation
  yt(1,t)=-log(rand(1,1,'u'))*ths(ct(t))(1)+as(ct(t))(1);  // data generation
  yt(2,t)=-log(rand(1,1,'u'))*ths(ct(t))(2)+as(ct(t))(2);  // data generation
end

// generation of prior classified data
yi=[]; ni=5;                            // no. of data from each comp.
for i=1:nc
  j=find(ct==i);                        // data from component i
  ji=j(10+(1:ni));                      // selected from data sample
  yi=[yi [yt(:,ji); i*ones(1,ni)]];     // stored in yi
end

// RESULTS
set(figure(1),'position',[140 50 500 500])
set(gcf(),'background',8);
plot(yt(1,:),yt(2,:),'.')
title('Data')

set(figure(2),'position',[660 50 500 500])
set(gcf(),'background',8);
plot(ct,'.')
title('Clusters')
set(gca(),'data_bounds',[1 nd .9 nc+.1])

// SAVING THE DATA SAMPLE (the name is compulsory)
als=list();
als=pp; ncs=nc;
save('_data/datExpEx','yt','zt','ct','yi','ths','as','als','ncs')
